namespace DrawingLayer
{
    private ref class Renderer : public IRenderer
    {
    private:
        int	m_targetWidth;
        int m_targetHeight;
        bool m_drawingInProcess;
        bool m_motionBlurEnabled;
        float m_motionBlurAngle;
        float m_motionBlurDeviation;

        SurfaceImageSource^	                m_imageSource;
        ImageBrush^				            m_imageBrush;

        ComPtr<ID2D1Factory1>               m_d2dFactory;
        ComPtr<ID3D11Device>				m_d3dDevice;
        ComPtr<ID2D1Device>					m_d2dDevice;
        ComPtr<ID3D11DeviceContext>			m_d3dContext;
        ComPtr<ID2D1DeviceContext>			m_d2dContext;
        ComPtr<IDXGIDevice>					m_dxgiDevice;
        ComPtr<ISurfaceImageSourceNative>	m_imageSourceNative;

        ComPtr<ID2D1Bitmap1>                m_inputSource;
        ComPtr<ID2D1Effect>                 m_blurEffect;

    internal:
        Renderer();

    private:
        void PrepareSurface(ID2D1Bitmap1** ppBitmap);
        D2D1_COLOR_F GetD2DColor(Color color);

    public:
        virtual property int TargetWidth { int get(); }
        virtual property int TargetHeight { int get(); }
        virtual property ImageBrush^ TargetBrush { ImageBrush^ get(); }
        virtual property bool EnableMotionBlur { bool get(); void set(bool value); }
        virtual property float MotionBlurAngle { float get(); void set(float value); }
        virtual property float MotionBlurDeviation { float get(); void set(float value); }

    public:
        virtual void Reset(int width, int height);
        virtual void BeginDraw();
        virtual void EndDraw();
        virtual void Clear(Color color);
        virtual IShapeGeometry^ CreateGeometry();
        virtual void DrawGeometry(IShapeGeometry^ geometry, Color color, float width);
        virtual void FillGeometry(IShapeGeometry^ geometry, Color color);
    };
}
