#include "pch.h"
#include "Interfaces.h"
#include "ShapeGeometry.h"

using namespace DrawingLayer;

ShapeGeometry::ShapeGeometry(ID2D1PathGeometry1* geometry) : m_d2dGeometry(geometry)
{
}

D2D_POINT_2F ShapeGeometry::GetD2DPoint(Point point)
{
	D2D_POINT_2F d2dPoint = { point.X, point.Y };
	return d2dPoint;
}

void ShapeGeometry::BeginFigure(Point start, bool isFilled)
{
	ThrowIfFailed(m_d2dGeometry->Open(&m_d2dSink));
	m_d2dSink->BeginFigure(GetD2DPoint(start), isFilled ? D2D1_FIGURE_BEGIN_FILLED : D2D1_FIGURE_BEGIN_HOLLOW);
}
void ShapeGeometry::EndFigure(bool close)
{
	m_d2dSink->EndFigure(close ? D2D1_FIGURE_END_CLOSED : D2D1_FIGURE_END_OPEN);
	ThrowIfFailed(m_d2dSink->Close());
	m_d2dSink.Reset();
}
void ShapeGeometry::AddLine(Point point)
{
	m_d2dSink->AddLine(GetD2DPoint(point));
}
void ShapeGeometry::Fill(ID2D1DeviceContext* d2dContext, D2D1_COLOR_F color)
{
	ComPtr<ID2D1SolidColorBrush> brush;
	ThrowIfFailed(d2dContext->CreateSolidColorBrush(color, &brush));
	d2dContext->FillGeometry(m_d2dGeometry.Get(), brush.Get());
}
void ShapeGeometry::Draw(ID2D1DeviceContext* d2dContext, D2D1_COLOR_F color, float width)
{
	ComPtr<ID2D1SolidColorBrush> brush;
	ThrowIfFailed(d2dContext->CreateSolidColorBrush(color, &brush));
	d2dContext->DrawGeometry(m_d2dGeometry.Get(), brush.Get(), width);
}