﻿using DrawingLayer;
using System;
using Windows.Foundation;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;

// The Templated Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234235

namespace Gauge {

    public sealed class Gauge : Control {
        static readonly DependencyProperty ValueProperty = DependencyProperty.Register("Value", 
            typeof(double), typeof(Gauge), new PropertyMetadata(0.0, ValuePropertyChanged));
        
        public double Value {
            get { return (double)GetValue(ValueProperty); }
            set { SetValue(ValueProperty, value); }
        }

        public static void ValuePropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e) {
            Gauge gauge = d as Gauge;
            if (gauge != null)
                gauge.renderer.EnableMotionBlur = true;
        }

        readonly IRenderer renderer;
        Border rendererSurface;
        IShapeGeometry arrow;
        
        public Gauge() {
            this.renderer = DrawingFactory.CreateRenderer();
            this.DefaultStyleKey = typeof(Gauge);
            this.SizeChanged += Gauge_SizeChanged;
            CompositionTarget.Rendering += CompositionTarget_Rendering;
        }
        void CompositionTarget_Rendering(object sender, object e) {
            Render();
        }
        void Gauge_SizeChanged(object sender, SizeChangedEventArgs e) {
            renderer.Reset((int)e.NewSize.Width, (int)e.NewSize.Height);
            rendererSurface.Background = renderer.TargetBrush;            
        }
        void Render() {
            double angle = Math.PI * Value / 1000 - Math.PI / 2.0;
            IShapeGeometry geometry = CreateArrowGeometry(angle);
            renderer.BeginDraw();
            renderer.MotionBlurAngle = (float)(angle / Math.PI * 180);
            renderer.MotionBlurDeviation = 5.0f;
            renderer.Clear(Colors.Transparent);
            renderer.FillGeometry(arrow, ColorHelper.FromArgb(0xFF, 0xD3, 0x65, 0xF8));
            renderer.DrawGeometry(arrow, ColorHelper.FromArgb(0xFF, 0xD3, 0xAA, 0xF8), 3.0f);
            renderer.EndDraw();
            if (renderer.EnableMotionBlur)
                renderer.EnableMotionBlur = false;
            
        }
        IShapeGeometry CreateArrowGeometry(double angle) {
            Point center = new Point(ActualWidth / 2.0, 0);
            double radius = Math.Min(ActualWidth / 2.0, ActualHeight) * 0.8;
            double arrowSection = Math.PI * radius / 5000;
            arrow = renderer.CreateGeometry();
            arrow.BeginFigure(RadialToCartesian(center, 80, angle - arrowSection / 2.0), true);
            arrow.AddLine(RadialToCartesian(center, radius, angle));
            arrow.AddLine(RadialToCartesian(center, 80, angle + arrowSection / 2.0));
            arrow.EndFigure(true);
            return arrow;
        }        
        Point RadialToCartesian(Point center, double radius, double angle) {
            return new Point(Math.Sin(angle) * radius + center.X, Math.Cos(angle) * radius + center.Y);
        }
        protected override void OnApplyTemplate() {
            base.OnApplyTemplate();
            rendererSurface = (Border)GetTemplateChild("RendererSurface");
        }
    }
}
